@@ -17,9 +17,16 @@ import os
 from time import sleep
 import psutil
 from multiprocessing import Process, Queue
+import threading
 
 import pickle
 
+canLoadScenedetect = 1
+try:
+    from scenedetect.video_manager import VideoManager
+except ImportError as e:
+    canLoadScenedetect = 0
+
 # baseUIClass, baseUIWidget = uic.loadUiType("mainwindow.ui")
 
 
@@ -43,13 +50,14 @@ class window(QMainWindow, Ui_qencoder):
     configpath = os.path.join(confighome, 'qencoder.qec')
 
     def __init__(self, *args, **kwargs):
+        global canLoadScenedetect
+        self.canLoadScenedetect = canLoadScenedetect
         QMainWindow.__init__(self, *args, **kwargs)
         self.setupUi(self)
         self.inputFileChoose.clicked.connect(self.inputFileSelect)
         self.outputFileChoose.clicked.connect(self.outputFileSelect)
         self.pushButton_vmafmodel.clicked.connect(self.inputVmafSelect)
         self.label_audio.setEnabled(0)
-        self.spinBox_quality.setValue(26)
         enable_slot = partial(self.audioEnableState, self.checkBox_audio)
         disable_slot = partial(self.audioDisableState, self.checkBox_audio)
         self.checkBox_audio.stateChanged.connect(lambda x: enable_slot() if x else disable_slot())
@@ -86,6 +94,7 @@ class window(QMainWindow, Ui_qencoder):
         self.actionOpen_Queue.triggered.connect(self.openQueueFrom)
         self.actionSave_Preset.triggered.connect(self.savePresetAs)
         self.actionOpen_Preset.triggered.connect(self.openPresetFrom)
+        self.actionReset_All_Settings.triggered.connect(self.resetAllSettings)
         self.pushButton_save.setEnabled(0)
         self.pushButton_save.clicked.connect(self.saveToQueue)
         self.tabWidget.currentChanged[int].connect(self.setCustomText)
@@ -143,12 +152,22 @@ class window(QMainWindow, Ui_qencoder):
             print("Unable to load existing preset at: " + str(self.configpath) + ".")
             print("Possibly the first time you have run this, corrupted, or an older version")
             print("Do not report this")
+            self.enableCropping()
+            self.enableRescale()
+            self.enableDisableVmaf()
+            self.enableDisableGoodSplit()
         # self.speedButton.changeEvent.connect(self.setSpeed)
 
     def enableDisableGoodSplit(self):
         state = self.checkBox_lessshitsplit.isChecked()
         self.label_split.setEnabled(not state)
         self.spinBox_split.setEnabled(not state)
+        if (self.canLoadScenedetect == 0):
+            self.checkBox_lessshitsplit.setEnabled(0)
+            self.checkBox_lessshitsplit.setChecked(1)
+            self.label_split.setEnabled(0)
+            self.spinBox_split.setEnabled(0)
+
 
     def enableDisableVmaf(self):
         state = self.checkBox_vmaf.isChecked()
@@ -211,8 +230,8 @@ class window(QMainWindow, Ui_qencoder):
         self.currentlyRunning = 0
         if success:
             self.pushButton.setEnabled(1)
-            self.pushButton.setText("Finalize")
             self.label_status.setText("Encoding complete!")
+            self.finalizeEncode()
         else:
             self.pushButton.setEnabled(1)
             self.pushButton.setStyleSheet("color: red; background-color: white")
@@ -226,6 +245,17 @@ class window(QMainWindow, Ui_qencoder):
     def updateQueuedStatus(self, queueString):
         self.label_queueprog.setText(queueString)
 
+    def resetAllSettings(self):
+        buttonReply = QMessageBox.question(self, 'Factory reset all settings?',
+                                                 "Clicking yes will cause the program to close and reset all settings. You may lose any existing encodes.", QMessageBox.Yes | QMessageBox.No, QMessageBox.No)
+        if buttonReply != QMessageBox.Yes:
+            return
+        else:
+            os.remove(Path(self.configpath))
+            if not sys.platform.startswith('win'):
+                os.killpg(0, signal.SIGTERM)
+            sys.exit()
+
     def openPresetFrom(self):
         filename = QFileDialog.getOpenFileName(filter = "Qencoder encoder config (*.qec)")
         newlist = []
@@ -598,7 +628,7 @@ class window(QMainWindow, Ui_qencoder):
         if (self.spinBox_maxkfdist.value() > 0):
             vparams += " --kf-max-dist=" + str(self.spinBox_maxkfdist.value())
         if (self.comboBox_encoder.currentIndex() < 2):
-            vparams += " --tile-columns=2 --tile-rows=1 --cpu-used=" + str(self.spinBox_speed.value())
+            vparams += " --tile-columns=1 --tile-rows=0 --cpu-used=" + str(self.spinBox_speed.value())
         else:
             vparams += " --codec=vp8 --cpu-used=" + str(self.spinBox_speed.value())
 
@@ -732,14 +762,13 @@ class window(QMainWindow, Ui_qencoder):
                 'better_split' : self.checkBox_lessshitsplit.isChecked(), 'cpuused' : self.spinBox_speed.value(),
                 'unsafe_split' : self.checkBox_unsafeSplit.isChecked()
         }
-        args['temp'] = Path(str(os.path.splitext(self.outputPath.text())[0]) + "/temp_" + str(os.path.splitext(self.outputPath.text())[1]))
+        args['temp'] = Path(str(os.path.dirname(self.outputPath.text())) + "/temp_" + str(os.path.basename(self.outputPath.text())))
 
-        if (self.checkBox_vmaf.isChecked()):
-            args['vmaf_steps'] = self.spinBox_vmafsteps.value()
-            args['min_cq'] = self.spinBox_minq.value()
-            args['max_cq'] = self.spinBox_maxq.value()
-            args['vmaf_target'] = self.doubleSpinBox_vmaf.value()
-            args['vmaf_path'] = self.label_vmafpath.text()
+        args['vmaf_steps'] = self.spinBox_vmafsteps.value()
+        args['min_cq'] = self.spinBox_minq.value()
+        args['max_cq'] = self.spinBox_maxq.value()
+        args['vmaf_target'] = self.doubleSpinBox_vmaf.value()
+        args['vmaf_path'] = self.label_vmafpath.text()
 
         if (self.checkBox_minsplit.isChecked()):
             args['min_split_dist'] = 0
@@ -754,7 +783,12 @@ class window(QMainWindow, Ui_qencoder):
             args['br'] = self.spinBox_boost.value()
         if (self.comboBox_encoder.currentIndex() >= 1):
             args['encoder'] = 'vpx'
-            args['better_split'] = False
+
+        args['temp_str'] = str(args['temp']).replace("'", "'\"'\"'")
+        args['input_file_str'] = str(args['input_file']).replace("'", "'\"'\"'")
+        args['output_file_str'] = str(args['output_file']).replace("'", "'\"'\"'")
+        print(args['temp_str'])
+
         return args
 
     def encodeVideoQueue(self):
@@ -811,6 +845,7 @@ class window(QMainWindow, Ui_qencoder):
         self.actionOpen_Queue.setEnabled(0)
         self.actionSave_Preset.setEnabled(0)
         self.actionOpen_Preset.setEnabled(0)
+        self.actionReset_All_Settings.setEnabled(0)
         self.label_3.setEnabled(0)
         self.label_threads.setEnabled(0)
         self.spinBox_threads.setEnabled(0)
@@ -821,6 +856,7 @@ class window(QMainWindow, Ui_qencoder):
         self.textEdit_audiocmd.setEnabled(0)
         self.textEdit_ffmpegcmd.setEnabled(0)
         self.currentlyRunning = True
+        self.pushButton.setEnabled(1)
         self.pushButton.setText("Cancel")
         self.pushButton_encQueue.setEnabled(0)
         self.pushButton_save.setEnabled(0)
@@ -997,6 +1033,7 @@ class window(QMainWindow, Ui_qencoder):
         self.actionOpen_Queue.setEnabled(1)
         self.actionSave_Preset.setEnabled(1)
         self.actionOpen_Preset.setEnabled(1)
+        self.actionReset_All_Settings.setEnabled(1)
         self.checkBox_cropping.setEnabled(1)
         self.checkBox_rescale.setEnabled(1)
         self.checkBox_unsafeSplit.setEnabled(1)
@@ -1006,6 +1043,7 @@ class window(QMainWindow, Ui_qencoder):
         self.enableDisableGoodSplit()
         print("Enabled all buttons, returning program to normal")
 
+
 class EncodeWorker(QtCore.QObject):
     updateStatusProgress = QtCore.pyqtSignal(str, int)
     updateQueuedStatus = QtCore.pyqtSignal(str)
@@ -1027,7 +1065,7 @@ class EncodeWorker(QtCore.QObject):
         av1an.main_thread(self)
         print("\n\nEncode completed for " + str(dictargs['input_file']) + " -> " + str(dictargs['output_file']))
 
-    def internalRunner(self):
+    def runInternal(self):
         print("Running")
         if (len(self.argdat) > 1):
             self.q.put([1, "Encoding video 1/" + str(len(self.argdat))])
@@ -1064,14 +1102,15 @@ class EncodeWorker(QtCore.QObject):
                     # If that doesn't work we can try shutting down the "other" way
                     os.system("shutdown now -h")
 
+
     def run(self):
-        t = Process(target = self.internalRunner)
+        t = threading.Thread(target=self.runInternal)
         t.start()
         while (t.is_alive()):
             sleep(0.05)
             if(self.window.killFlag):
                 print("Killing all children processes. Hopefully this works.")
-                parent = psutil.Process(t.pid)
+                parent = psutil.Process(os.getpid())
                 for child in parent.children(recursive=True):  # or parent.children() for recursive=False
                     child.kill()
                 parent.kill()
@@ -1086,4 +1125,4 @@ class EncodeWorker(QtCore.QObject):
                     self.encodeFinished.emit(qdat[1])
             except Exception as e:
                 pass
-        self.q.put([0, "Encode completed!", 100])
+        self.updateStatusProgress.emit("Encode completed!", 100)
